<?php
/*
Plugin Name:        Quttera Web Malware Scanner
Plugin URI:         http://quttera.com
Description:        Web malware and vulnerability exploits scanner for WordPress.
Version:            1.1.1
Author:             Quttera team
Author URI:         http://quttera.com/
License:            GNU General Public License v2
*/

if(!function_exists('add_action'))
{
    exit(0);
}

define('QUTTERA_WM_SCANNER','quttera_wm_scanner');
define('QUTTERA_WM_SCANNER_VERSION','1.1.1');
define('QUTTERA_URL',plugin_dir_url( __FILE__ ));

add_action( 'admin_enqueue_scripts', 'quttera_style', 1 );
function quttera_style() {

echo '<link rel="stylesheet" href="'.QUTTERA_URL.'quttera_css.css" type="text/css" media="all" />';

}

/* qtr_wm_scanner */
@ini_set( 'max_execution_time', 300 );

/**
 * Set up the menu item and register with hooks to print JS and help.
 */
function qtr_setup_scanner_menu() {
    
    $page_hook = add_menu_page( 'Quttera Web Malware Scanner', 
                                'Quttera Scanner',
                                'manage_options',
                                'quttera_wm_scanner', 
                                'qtr_admin_page'
                            ); 
    
    if ( $page_hook ) 
    {
        add_action( "admin_print_styles-$page_hook", 'add_thickbox' );
        add_action( "admin_footer-$page_hook", 'qtr_load_admin_scripts' );
    }
}

add_action( 'admin_menu', 'qtr_setup_scanner_menu' );


/**
 * Print scripts that power paged scanning and diff modal.
 */
function qtr_load_admin_scripts() { ?>

<script type="text/javascript">
    jQuery(document).ready(function($) {
        $('#run-scanner').click( function() {
            var url             = $('#url_name').val();
            var qtr_srv_name     = $('#qtr_srv_name').val();
            
            $.ajaxSetup({
                type: 'POST',
                url: ajaxurl,
                complete: function(xhr,status) {
                    if ( status != 'success' ) {
                        //alert("Failed to communicate with WP");
                    }
                }
            });
            run_scan(url,qtr_srv_name);
            return false;
        });
    }); 

    run_scan = function(this_url,qtr_url,level) {

        if( !level )
        {
            hide_all();
            var curr_time = new Date().getTime();
            show_investigation_status( {    "state" : "starting", 
                                            "age"   : curr_time,
                                            "url"   : this_url
                                       });
        }
        
        jQuery.ajax({
            data: {
                action: 'scanner-run_scan',
                _this: this_url,
                _qtr_url: qtr_url
                //_nonce: qtr_wm_scanner_nonce
            }, success: function(r) {
                var res = jQuery.parseJSON(r);
                var state = res.content.state.toLowerCase();
                //alert(res);
                //alert(res.status);
                //alert("Content: " + res.content);
                //alert("State: " + res.content.state );
                
                if ( state == 'new' )
                {
                    show_investigation_status({ "state" : "waiting for web crawler", 
                                                "age"   : res.content.age,
                                                "url"   : res.content.url });

                    run_scan(this_url,qtr_url,1); //recursive call
                }
                else if( state == 'download')
                {
                    show_investigation_status({ "state" : "website content is being downloaded for investigation", 
                                                "age"   : res.content.age,
                                                "url"   : res.content.url });

                    run_scan(this_url,qtr_url,1); //recursive call
                }
                else if( state =='downloaded' )
                {
                    show_investigation_status({ "state" : "website content has been downloaded and is waiting for scanner", 
                                                "age"   : res.content.age,
                                                "url"   : res.content.url });

                    run_scan(this_url,qtr_url,1); //recursive call
                }
                else if( state =='scan' || state =='scanned' )
                {
                    show_investigation_status({ "state" : "website content is being scanned", 
                                                "age"   : res.content.age,
                                                "url"   : res.content.url });

                    run_scan(this_url,qtr_url,1); //recursive call
                }
                else if( state == 'clean' )
                {
                    show_investigation_report(res.content.report);
                }
                else if( state=='potentially suspicious' || state=='potentially unsafe' )
                {
                    show_investigation_report(res.content.report);
                }
                else if (state=='suspicious' || state=='unsafe')
                {
                    show_investigation_report(res.content.report);                
                }
                else if (state=='malicious')
                {
                    show_investigation_report(res.content.report);                
                }
                else
                {
                    show_investigation_error(res.content);
                }                
            }
        });
    };

    show_investigation_error = function( status ){
        jQuery('#investigation_error').empty();
            var str =   "State: "     + status.state     + "</br>" +
                        "Time: "     + status.age     + "</br>" +
                        "Url: "     + status.url;
        //alert("Status: " + str );
        hide_all();
        jQuery('#investigation_error').append("<p>" + str + "</p>");
        jQuery('#investigation_error').show();
    }
    
    show_investigation_status = function ( status ){
        hide_all();
        var urlDate     = new Date();
        var currentdate = urlDate.toLocaleString();

        jQuery('#investigation_progress').empty();
        var str = "State: <b> " + status.state  + "</b></br>" +
                  "Time: "  + currentdate   + "</br>" +
                  "Url: "   + status.url;
        //alert("Status: " + str );
        
        jQuery('#investigation_progress').append("<p>" + str + "</p>");
        jQuery('#investigation_progress').show();
    };
    
    show_investigation_report = function ( report ){
        hide_all();
        jQuery('#investigation_result').empty();
        jQuery('#investigation_result').append('<H2>Website Investigation Report</H2>');
        jQuery('#investigation_result').append('<a href="http://quttera.com/article/about-quttera-malware-scan-report" target="_blank">Understanding security reports</a>');
        
        var clean_files             = 0;
        var pot_suspicious_files    = 0;
        var suspicious_files        = 0;
        var malicious_files         = 0;
        
        for( var i = 0; i < report.length; i ++ )
        {
            var threat = report[i].threat.toLowerCase();
            if( threat == "malicious" ){
                malicious_files         += 1;
            }else if( threat == "suspicious" ){
                suspicious_files        +=1;
            }else if( threat == "potentially suspicious"){
                pot_suspicious_files    += 1;
            }else{
                clean_files             += 1;
            }
        }
        
        var summary =   "<table>" +
                        "<tr><td align='left'><font color='green'><b>Clean files: </b></font></td>"+
                            "<td align='left'><font color='green'><b>"  + clean_files + "</b></font></td>" +
                            "<td align='left'><font color='green'><b>"  + "<input type='checkbox' id='clean_cb' value=''>" + "</b></font></td></tr>" +
                        "<tr><td align='left'><font color='orange'><b>Potentially Suspicious files: </b></font></td>"+
                            "<td align='left'><font color='orange'><b>" + pot_suspicious_files + "</b></font></td>" +
                            "<td align='left'><font color='green'><b>"  + "<input type='checkbox' id='pot_susp_cb' value=''>" + "</b></font></td></tr>" +
                        "<tr><td align='left'><font color='red'><b>Suspicious files: </b></font></td>"+
                            "<td align='left'><font color='red'><b>" + suspicious_files + "</b></font></td>" +
                            "<td align='left'><font color='green'><b>"  + "<input type='checkbox' id='susp_cb' value=''>" + "</b></font></td></tr>" +
                        "<tr><td align='left'><font color='#780000'><b>Malicious files: </b></font></td>"+
                            "<td align='left'><font color='#780000'><b>"    + malicious_files + "</b></font></td>" +
                            "<td align='left'><font color='green'><b>"  + "<input type='checkbox' id='mal_cb' value=''>" + "</b></font></td></tr>" +
                        "</table>" + 
                        "<hr/>";
        
        jQuery('#investigation_result').append(summary);
        document.getElementById('clean_cb').checked=true;
        document.getElementById('pot_susp_cb').checked=true;
        document.getElementById('susp_cb').checked=true;
        document.getElementById('mal_cb').checked=true;
        
        for( var i = 0; i < report.length; i ++ )
        {
            var color = "green";
            var tab_class = 'clean_tab';
            var threat = report[i].threat.toLowerCase();
            if( threat == "malicious" ){
                color = "#780000";
                var tab_class = 'mal_tab';
            }else if( threat == "suspicious" ){
                color = "red";
                var tab_class = 'susp_tab';
            }else if( threat == "potentially suspicious"){
                color = "orange";
                var tab_class = 'pot_susp_tab';
            }

            var str = "<table>" +
                        "<tr><td align='left'><font color='"  + color +"'>Filename: </font></td>"+
                            "<td align='left'><font color='"  + color +"'><b>http://" + report[i].filename + "</b></font></td></tr>"+
                        "<tr><td align='left'><font color='"  + color +"'>Severity:   </font></td>"+
                            "<td align='left'><font color='" + color +"'>" +  report[i].threat  + "</font></td></tr>"+
                        "<tr><td align='left'><font color='"  + color +"'>Reason:   </font></td>"+
                            "<td align='left'><font color='"  + color +"'>" + report[i].reason   + "</font></td></tr>" +
                        "<tr><td align='left'><font color='"  + color +"'>Details:  </font></td>"+
                            "<td align='left'><font color='"  + color +"'>" + report[i].details  + "</font></td></tr>";
 
            if( report[i].offset > 0)
            {
                str += "<tr><td align='left'><font color='"  + color +"'>Offset:  </font></td>"+
                            "<td align='left'><font color='"  + color +"'>" + report[i].offset  + "</font></td></tr>";
            }
            
            if( report[i].dump)
            {
                str += "<tr><td align='left'><font color='"  + color +"'>Threat dump:  </font></td>"+
                            "<td align='left'><font color='"  + color +"'>" + "<TEXTAREA disabled readonly COLS=65 ROWS=6  width='auto'>" + report[i].dump  + "</TEXTAREA></div>" + "</font></td></tr>";
            }
            
            str += "<tr><td align='left'><font color='"  + color +"'>Filesize: </font></td>"+
                       "<td align='left'><font color='" + color +"'>" + report[i].filesize + "</font></td></tr>";

            if( report[i].filetype)
            {
                str += "<tr><td align='left'><font color='"  + color +"'>File type:  </font></td>"+
                            "<td align='left'><font color='"  + color +"'>" + report[i].filetype  + "</font></td></tr>";
            }

            str += "<tr><td align='left'><font color='"  + color +"'>MD5:        </font></td>" +
                        "<td align='left'><font color='"  + color +"'>" +     report[i].md5    + "</font></td></tr>" +
                   "<tr><td align='left'><font color='"  + color +"'>Scan duration: </font></td>"+
                        "<td align='left'><font color='"  + color +"'>" + report[i].scantime + "</font></td></tr>" +
                   "</table>";

            jQuery('#investigation_result').append("<div class='" + tab_class+ "' align='left'>"+ str + "<hr/></div>" + "</font></p>");
        }
        jQuery('#investigation_result').show();
        
        jQuery('#clean_cb').click(function(){
           if(jQuery('#clean_cb:checked').length){
             jQuery('.clean_tab').show();
             } else {
             jQuery('.clean_tab').hide(); 
             }
        });
        jQuery('#pot_susp_cb').click(function(){
           if(jQuery('#pot_susp_cb:checked').length){
             jQuery('.pot_susp_tab').show();
             } else {
             jQuery('.pot_susp_tab').hide(); 
             }
        });
        jQuery('#susp_cb').click(function(){
           if(jQuery('#susp_cb:checked').length){
             jQuery('.susp_tab').show();
             } else {
             jQuery('.susp_tab').hide(); 
             }
        });
        jQuery('#mal_cb').click(function(){
           if(jQuery('#mal_cb:checked').length){
             jQuery('.mal_tab').show();
             } else {
             jQuery('.mal_tab').hide(); 
             }
        });
     
    };
    
    
    hide_all = function(){
        jQuery('#investigation_result').hide();
        jQuery('#investigation_error').hide();
        jQuery('#investigation_progress').hide();    
    }


    
</script>
<?php
}


function qtr_wm_scanner_ajax_run_scan()
{
    if(!current_user_can('manage_options'))
    {
        wp_die(__('You do not have sufficient permissions to access this page.') );
    }

    //check_ajax_referer( 'qtr_wm_scanner-scan' );
    $this_url = $_POST['_this'];        /* domain name of this server */
    $qtr_url  = $_POST['_qtr_url'];        /* quttera investigation server name */
    
    
    if( empty($this_url) )
    {
        $this_url = qtr_get_domain_name();
        //echo json_encode( array('content' => array('state' => 'Provided domain name is empty',"url" => "undefiend","age" => time() ) ));
        //exit;
    }
    else if( empty($qtr_url) )
    {
        /*
        echo json_encode( array( 'content' => array(     "state" => "Provided investigation server name is invalid", 
                                                        "url"     => $this_url,
                                                        "age" => time() 
                                                    ) 
                                ) 
                        );
        exit;
        */
        $qtr_url = "http://wp.quttera.com";
    }
    
    if( strpos($this_url, "://" ) != false )
    {
        $parse = parse_url($this_url);
        $this_url = $parse['host'];
    }
    
    usleep(1000000); //sleep for a second
    
    $investigation_url =  $qtr_url . "/wp_scan/" . $this_url;

    $output = file_get_contents($investigation_url);
    
    if( empty($output) )
    {
        $output = file_get_contents($investigation_url);
    
        if( $output == false )
        {
            echo json_encode( array( 'content' => array( "state" => "Failed to access investigation server [ " . $qtr_url . " ]",
                                                         "age" => time(),
                                                         "url" => $this_url,
                                                         "img" => plugins_url( 'loader.gif', __FILE__ )) 
                                    ) 
                            );
            exit;
        }
    }
    
    $output = json_decode($output);
    
    //if state is not finished sleep for a second
    echo json_encode( array( 'content' => $output ) );
    exit;
}

add_action( 'wp_ajax_scanner-run_scan', 'qtr_wm_scanner_ajax_run_scan' );

/**
 * add_management_page callback
 */
function qtr_admin_page(){
    // non-ajax scan form processing
    echo '<table>';
    echo '<tr>';
    echo '<td><img src="' . plugins_url( 'quttera_icon.png', __FILE__ ) . '" height="30px" width="30px"/></td>';
    echo '<td><h2>Quttera Web Malware Scanner for WordPress</h2></td>';
    echo '</tr>';
    echo '</table>';
    
    if(!current_user_can('manage_options'))
    {
        wp_die(__('You do not have sufficient permissions to access this page.') );
    }

    show_admin_page();
}


function qtr_get_domain_name(){
    $domain_name = network_site_url( '/' );
    $parse = parse_url($domain_name);
    return $parse['host']; // prints 'google.com'
}


/**
 * Display scan initiation form and any stored results.
 */
function show_admin_page() {
    global $wp_version;
    delete_transient( 'exploitscanner_results_trans' );
    delete_transient( 'exploitscanner_files' );
    $results = get_option( 'exploitscanner_results' );
?>
    <div style="width:500px;float:left;"><p><strong>Quttera Web Malware Scanner</strong> investigates content of your web-site/blog and 
    detects: 
        <ul>
        <li>* <b>malicious code</b></li>
        <li>* <b>vulnerability exploits(shell-codes)</b></li>
        <li>* <b>JavaScript obfuscation techniques</b></li>
        <li>* <b>hidden iframes</b></li>
        <li>* <b>unconditional re-directions</b></li> 
        </ul>
    and other potentially unsafe content if there is such.</p>
    <p>The investigation is performed by Quttera engines and it is run on our remote servers. </p>
	<p>It does not impact performance of your web-site/blog and does not modify any of the files.</p>
    <p>For more information visit <a href="http://quttera.com" target="_blank">quttera.com</a> or send email to <b>contactus@quttera.com</b>
    </p>
    </div>
    <div id="help_block" style="margin-top:-49px;margin-right:10px;float:right;width:auto;padding:10px;background:#f7f7f7;border:1px solid #c6c6c6;">
         <p><b>Help links</b></p>
         <p><a href="http://quttera.com/faq" target="_blank">FAQ section</a></p>
         <p><a href="http://quttera.com/article/about-quttera-malware-scan-report" target="_blank">About security reports</a></p>
         <p><b>Useful resources</b></p>
         <p><a href="http://quttera.com/website-anti-malware-monitoring" target="_blank">Periodic scan service</a></p>
         <p><a href="http://quttera.com/lists/malware-free-websites-category" target="_blank">Featured websites database</a></p>
         <p><a href="http://quttera.com/lists/cleans" target="_blank">Clean websites</a></p>
         <p><a href="http://quttera.com/lists/potentially_suspicious" target="_blank">Potentially Suspicious websites database</a></p>
         <p><a href="http://quttera.com/lists/suspicious" target="_blank">Suspicious websites database</a></p>
         <p><a href="http://quttera.com/lists/malicious" target="_blank">Malicious websites database</a></p>
 </div>
 <div style="margin-bottom: 14px;clear:both;">
  <a class="btn btn-large btn-success" style="text-decoration: none;" target="_blank" href="http://quttera.com/anti-malware-website-monitoring-signup">Sign up for website anti-malware monitoring! <i class="icon-white icon-ok"></i></a>
 <a class="btn btn-large btn-info" style="text-decoration: none;" target="_blank" href="http://quttera.com/malware-free-websites-category-signup">Add your website to malware-free directory! <i class="icon-white icon-ok"></i></a>
 </div>
 <div style="margin-right:10px;clear:both;background:#f7f7f7;border:1px solid #c6c6c6;padding:10px;">
	<p><b>Did you know?</b> If your blog/ website is clean from malware it can be proudly listed in our <a href="http://quttera.com/lists/malware-free-websites-category" target="_blank">Exploits-free websites</a> category. <a href="http://quttera.com/about-malware-free-websites-category" target="_blank">Find out how!</a></p>
	<p><b>Was this plugin helpful?</b> Please leave feedback on <a href="http://wordpress.org/support/view/plugin-reviews/quttera-web-malware-scanner" target="_blank">Plugin reviews page</a></p>
	<p style="font-size:10px;"><center>Please join Quttera community on <a href="https://www.facebook.com/Quttera" target="_blank">Facebook</a> | <a href="https://twitter.com/quttera" target="_blank">Twitter</a> | <a href="https://www.youtube.com/user/quttera" target="_blank">YouTube</a> and stay updated with more security features to come.</center></p>
 </div>
    <form style="width:603px;" action="<?php admin_url( 'tools.php?page=quttera_wm_scanner' ); ?>" method="post">
        <?php wp_nonce_field( /*action*/'qtr_wm_scanner-scan_url' ); ?>
        <input type="hidden" name="action" value="scan" />
        <table class="form-table">
            <tr>
                <th scope="row"><label for="url_name">Domain name:</label></th>
                <td><input type="text" size="20" id="url_name" name="url_name" value="<?php echo qtr_get_domain_name(); ?>" />
                    <span class="description">(Your domain name)</span>
                </td>
            </tr>
            <tr>
                <th scope="row"><label for="url_name">Quttera server name:</label></th>
                <td><input type="text" size="20" id="qtr_srv_name" name="qtr_srv_name" value="http://wp.quttera.com"/>
                    <span class="description">(Name of Quttera's investigation server)</span>
                </td>
            </tr>
        </table>
        <p class="submit">
            <input type="submit" id="run-scanner" class="button-primary" value="Scan my website"/>
        </p>
    </form>
    <div id="investigation_progress" style="display:none;margin:10px;padding:10px;background:#f7f7f7;border:1px solid #c6c6c6;text-align:center">
        <p><strong>Scanning your web site for web malware, vulnerability exploits and suspicious signs</strong></p>
        <p><span style="margin-right:5px"></span><img src="<?php echo plugins_url( 'loader.gif', __FILE__ ); ?>" height="16px" width="16px" alt="loading-icon" /></p>
    </div>

    <div id="investigation_error" style="display:none;margin:10px;padding:10px;background:#f7f7f7;border:1px solid #c6c6c6;text-align:center">
        <p><strong>Communication error occured. Try to scan later.</strong></p>
    </div>
    <div id="investigation_result" style="display:none;margin:10px;padding:10px;background:#f7f7f7;border:1px solid #c6c6c6;text-align:center">
    </div>
    <?php
}



/**
 * Activation callback.
 *
 * Add database version info. Set up non-autoloaded options as there is no need
 * to load results or clean core files on any page other than the exploit scanner page.
 */
function on_qtr_scanner_activation() {
    register_uninstall_hook( __FILE__, 'on_qtr_scanner_uninstall' );
}

register_activation_hook( __FILE__, 'on_qtr_scanner_activation' );

/**
 * Deactivation callback. Remove transients.
 */
function on_qtr_scanner_deactivate() {
    /* remove cached (runtime) parameters */
}

register_deactivation_hook( __FILE__, 'on_qtr_scanner_deactivate' );


/**
 * Uninstall callback. Remove all data stored by the plugin.
 */
function on_qtr_scanner_uninstall()
{
    /* removes all added options */
}

/**
 * Update routine to perform database cleanup and to ensure that newly
 * introduced settings and defaults are enforced.
 */
function on_qtr_scanner_admin_init() 
{
    on_qtr_scanner_activation();
}

/* admin_init is triggered before any other hook when a user access the admin area */
add_action( 'admin_init', 'on_qtr_scanner_admin_init' );

/* ?????? */
function qtr_wm_scanner_plugin_actions( $links, $file ) {
     if( $file == 'quttera/quttera_wm_scanner.php' && function_exists( "admin_url" ) ) {
        $settings_link = '<a href="' . admin_url( 'tools.php?page=quttera_wm_scanner' ) . '">' . __('Scanner Settings') . '</a>';
        array_unshift( $links, $settings_link ); // before other links
    }
    return $links;
}

add_filter( 'plugin_action_links', 'qtr_wm_scanner_plugin_actions', 10, 2 );

/* end of file */
